@@ -7,7 +7,7 @@
  * @date   2006-09-15
  * @brief  Xmp manipulation methods
  *
- * @author Copyright (C) 2006-2012 by Gilles Caulier
+ * @author Copyright (C) 2006-2014 by Gilles Caulier
  *         <a href="mailto:caulier dot gilles at gmail dot com">caulier dot gilles at gmail dot com</a>
  * @author Copyright (C) 2006-2012 by Marcel Wiesweg
  *         <a href="mailto:marcel dot wiesweg at gmx dot de">marcel dot wiesweg at gmx dot de</a>
@@ -45,8 +45,12 @@ bool KExiv2::canWriteXmp(const QString& filePath)
     catch( Exiv2::Error& e )
     {
         std::string s(e.what());
-        kDebug() << "Cannot check Xmp access mode using Exiv2 (Error #"
-                      << e.code() << ": " << s.c_str() << ")";
+        kError() << "Cannot check Xmp access mode using Exiv2 (Error #"
+                 << e.code() << ": " << s.c_str() << ")";
+    }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
     }
 
 #else
@@ -84,6 +88,10 @@ bool KExiv2::clearXmp() const
     {
         d->printExiv2ExceptionError("Cannot clear Xmp data using Exiv2 ", e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #endif // _XMP_SUPPORT_
 
@@ -112,6 +120,10 @@ QByteArray KExiv2::getXmp() const
 
         d->printExiv2ExceptionError("Cannot get Xmp data using Exiv2 ", e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #endif // _XMP_SUPPORT_
 
@@ -138,10 +150,14 @@ bool KExiv2::setXmp(const QByteArray& data) const
     catch( Exiv2::Error& e )
     {
         if (!d->filePath.isEmpty())
-            kDebug() << "From file " << d->filePath.toAscii().constData();
+            kError() << "From file " << d->filePath.toAscii().constData();
 
         d->printExiv2ExceptionError("Cannot set Xmp data using Exiv2 ", e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #else
 
@@ -195,34 +211,55 @@ KExiv2::MetaDataMap KExiv2::getXmpTagsDataList(const QStringList& xmpKeysFilter,
 
             // We apply a filter to get only the XMP tags that we need.
 
-            if (!invertSelection)
+            if (!xmpKeysFilter.isEmpty())
             {
-                if (xmpKeysFilter.contains(key.section('.', 1, 1)))
+                if (!invertSelection)
                 {
-                    if (it == metaDataMap.end())
-                        metaDataMap.insert(key, value);
-                    else
+                    if (xmpKeysFilter.contains(key.section('.', 1, 1)))
                     {
-                        QString v = *it;
-                        v.append(", ");
-                        v.append(value);
-                        metaDataMap.insert(key, v);
+                        if (it == metaDataMap.end())
+                        {
+                            metaDataMap.insert(key, value);
+                        }
+                        else
+                        {
+                            QString v = *it;
+                            v.append(", ");
+                            v.append(value);
+                            metaDataMap.insert(key, v);
+                        }
+                    }
+                }
+                else
+                {
+                    if (!xmpKeysFilter.contains(key.section('.', 1, 1)))
+                    {
+                        if (it == metaDataMap.end())
+                        {
+                            metaDataMap.insert(key, value);
+                        }
+                        else
+                        {
+                            QString v = *it;
+                            v.append(", ");
+                            v.append(value);
+                            metaDataMap.insert(key, v);
+                        }
                     }
                 }
             }
-            else
+            else // else no filter at all.
             {
-                if (!xmpKeysFilter.contains(key.section('.', 1, 1)))
+                if (it == metaDataMap.end())
                 {
-                    if (it == metaDataMap.end())
-                        metaDataMap.insert(key, value);
-                    else
-                    {
-                        QString v = *it;
-                        v.append(", ");
-                        v.append(value);
-                        metaDataMap.insert(key, v);
-                    }
+                    metaDataMap.insert(key, value);
+                }
+                else
+                {
+                    QString v = *it;
+                    v.append(", ");
+                    v.append(value);
+                    metaDataMap.insert(key, v);
                 }
             }
         }
@@ -233,6 +270,10 @@ KExiv2::MetaDataMap KExiv2::getXmpTagsDataList(const QStringList& xmpKeysFilter,
     {
         d->printExiv2ExceptionError("Cannot parse Xmp metadata using Exiv2 ", e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #else
 
@@ -258,6 +299,10 @@ QString KExiv2::getXmpTagTitle(const char* xmpTagName)
     {
         d->printExiv2ExceptionError("Cannot get Xmp metadata tag title using Exiv2 ", e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #else
 
@@ -281,6 +326,10 @@ QString KExiv2::getXmpTagDescription(const char* xmpTagName)
     {
         d->printExiv2ExceptionError("Cannot get Xmp metadata tag description using Exiv2 ", e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #else
 
@@ -317,6 +366,10 @@ QString KExiv2::getXmpTagString(const char* xmpTagName, bool escapeCR) const
         d->printExiv2ExceptionError(QString("Cannot find Xmp key '%1' into image using Exiv2 ")
                                     .arg(xmpTagName), e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #else
 
@@ -347,6 +400,10 @@ bool KExiv2::setXmpTagString(const char* xmpTagName, const QString& value, bool
     {
         d->printExiv2ExceptionError("Cannot set Xmp tag string into image using Exiv2 ", e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #else
 
@@ -358,7 +415,56 @@ bool KExiv2::setXmpTagString(const char* xmpTagName, const QString& value, bool
 
     return false;
 }
+bool KExiv2::setXmpTagString(const char* xmpTagName, const QString& value,
+                             KExiv2::XmpTagType type, bool setProgramName) const
+{
+#ifdef _XMP_SUPPORT_
 
+    if (!setProgramId(setProgramName))
+        return false;
+
+    try
+    {
+        const std::string &txt(value.toUtf8().constData());
+        Exiv2::XmpTextValue xmpTxtVal("");
+
+        if(type == KExiv2::NormalTag) // normal type
+        {
+            xmpTxtVal.read(txt);
+            d->xmpMetadata().add(Exiv2::XmpKey(xmpTagName),&xmpTxtVal);
+            return true;
+        }
+        if(type == KExiv2::ArrayBagTag) // xmp type = bag
+        {
+            xmpTxtVal.setXmpArrayType(Exiv2::XmpValue::xaBag);
+            xmpTxtVal.read("");
+            d->xmpMetadata().add(Exiv2::XmpKey(xmpTagName),&xmpTxtVal);
+        }
+        if(type == KExiv2::StructureTag) // xmp type = struct
+        {
+            xmpTxtVal.setXmpStruct();
+            d->xmpMetadata().add(Exiv2::XmpKey(xmpTagName),&xmpTxtVal);
+        }
+    }
+    catch( Exiv2::Error& e )
+    {
+        d->printExiv2ExceptionError("Cannot set Xmp tag string into image using Exiv2 ", e);
+    }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
+
+#else
+
+    Q_UNUSED(xmpTagName);
+    Q_UNUSED(value);
+    Q_UNUSED(setProgramName);
+
+#endif // _XMP_SUPPORT_
+
+    return false;
+}
 KExiv2::AltLangMap KExiv2::getXmpTagStringListLangAlt(const char* xmpTagName, bool escapeCR) const
 {
 #ifdef _XMP_SUPPORT_
@@ -394,6 +500,10 @@ KExiv2::AltLangMap KExiv2::getXmpTagStringListLangAlt(const char* xmpTagName, bo
         d->printExiv2ExceptionError(QString("Cannot find Xmp key '%1' into image using Exiv2 ")
                                     .arg(xmpTagName), e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #else
 
@@ -440,6 +550,10 @@ bool KExiv2::setXmpTagStringListLangAlt(const char* xmpTagName, const KExiv2::Al
     {
         d->printExiv2ExceptionError("Cannot set Xmp tag string lang-alt into image using Exiv2 ", e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #else
 
@@ -487,6 +601,10 @@ QString KExiv2::getXmpTagStringLangAlt(const char* xmpTagName, const QString& la
         d->printExiv2ExceptionError(QString("Cannot find Xmp key '%1' into image using Exiv2 ")
                                     .arg(xmpTagName), e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #else
 
@@ -544,6 +662,10 @@ bool KExiv2::setXmpTagStringLangAlt(const char* xmpTagName, const QString& value
     {
         d->printExiv2ExceptionError("Cannot set Xmp tag string lang-alt into image using Exiv2 ", e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #else
 
@@ -594,6 +716,10 @@ QStringList KExiv2::getXmpTagStringSeq(const char* xmpTagName, bool escapeCR) co
         d->printExiv2ExceptionError(QString("Cannot find Xmp key '%1' into image using Exiv2 ")
                                     .arg(xmpTagName), e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #else
 
@@ -637,6 +763,10 @@ bool KExiv2::setXmpTagStringSeq(const char* xmpTagName, const QStringList& seq,
     {
         d->printExiv2ExceptionError("Cannot set Xmp tag string Seq into image using Exiv2 ", e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #else
 
@@ -685,6 +815,10 @@ QStringList KExiv2::getXmpTagStringBag(const char* xmpTagName, bool escapeCR) co
         d->printExiv2ExceptionError(QString("Cannot find Xmp key '%1' into image using Exiv2 ")
                                     .arg(xmpTagName), e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #else
 
@@ -728,6 +862,10 @@ bool KExiv2::setXmpTagStringBag(const char* xmpTagName, const QStringList& bag,
     {
         d->printExiv2ExceptionError("Cannot set Xmp tag string Bag into image using Exiv2 ", e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #else
 
@@ -884,6 +1022,10 @@ QVariant KExiv2::getXmpTagVariant(const char* xmpTagName, bool rationalAsListOfI
         d->printExiv2ExceptionError(QString("Cannot find Xmp key '%1' into image using Exiv2 ")
                                     .arg(xmpTagName), e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #else
 
@@ -909,7 +1051,11 @@ bool KExiv2::registerXmpNameSpace(const QString& uri, const QString& prefix)
     }
     catch( Exiv2::Error& e )
     {
-        KExiv2Priv::printExiv2ExceptionError("Cannot register a new Xmp namespace using Exiv2 ", e);
+        Private::printExiv2ExceptionError("Cannot register a new Xmp namespace using Exiv2 ", e);
+    }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
     }
 
 #else
@@ -935,7 +1081,11 @@ bool KExiv2::unregisterXmpNameSpace(const QString& uri)
     }
     catch( Exiv2::Error& e )
     {
-        KExiv2Priv::printExiv2ExceptionError("Cannot unregister a new Xmp namespace using Exiv2 ", e);
+        Private::printExiv2ExceptionError("Cannot unregister a new Xmp namespace using Exiv2 ", e);
+    }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
     }
 
 #else
@@ -968,6 +1118,10 @@ bool KExiv2::removeXmpTag(const char* xmpTagName, bool setProgramName) const
     {
         d->printExiv2ExceptionError("Cannot remove Xmp tag using Exiv2 ", e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
 #else
 
@@ -1047,12 +1201,18 @@ KExiv2::TagsMap KExiv2::getXmpTagsList() const
         d->getXMPTagsListFromPrefix("iptc",           tagsMap);
         d->getXMPTagsListFromPrefix("iptcExt",        tagsMap);
         d->getXMPTagsListFromPrefix("plus",           tagsMap);
+        d->getXMPTagsListFromPrefix("mwg-rs",         tagsMap);
+        d->getXMPTagsListFromPrefix("dwc",            tagsMap);
         return tagsMap;
     }
     catch( Exiv2::Error& e )
     {
         d->printExiv2ExceptionError("Cannot get Xmp Tags list using Exiv2 ", e);
     }
+    catch(...)
+    {
+        kError() << "Default exception from Exiv2";
+    }
 
     return TagsMap();
 }
